# -*- coding: utf-8 -*-
from collections import defaultdict, OrderedDict


class EsfReport(object):
    """
    Representation of Report (»Durchführungsbericht«)
    """

    def __init__(self, programname, rows):
        self.rows = rows
        self.programmname = programname
        self.indicators = EsfReport.indicators_for(programname)
        self.female, self.male = self.determine_values_for_genders(rows)

    @classmethod
    def specific_indicators_for(cls, programname):
        """
        Specific indicators related to programs
        :param programname:
        :return:
        """
        programspecific_indicators = {
            "WASKIQ": ["b3_2", "po04"],
            "PWE": ["po02", "a2_1"],
            "SIB": ["po02", "a2_1"],
            "QE": ["c1_1"],
            "EC": ["c1_2"]
        }
        return programspecific_indicators.get(programname, [])

    @classmethod
    def general_indicators(cls):
        """
        Common indicators to all programs
        :return: indicators
        """
        return ["co01", "co02", "co03", "co04", "co05", "co06",
                "co07", "co08", "co09", "co10", "co11",
                "co15", "co16", "co17", "co18",
                "cr01", "cr02", "cr03", "cr04", "cr05"]

    @classmethod
    def indicators_for(cls, programname):
        """
        Indicators for all programms
        :param programname:
        :return:
        """
        return EsfReport.general_indicators() + EsfReport.specific_indicators_for(programname)

    @classmethod
    def keys(cls, progamname):
        return ["total", "enters"]

    def to_line(self, cols=None):
        if cols is None:
            cols = ["total", "male", "female"]
        d = self.to_dict()
        line = [d["total"]]
        fields = ["enters","exits"] + self.indicators
        for f in fields:
            for c in cols:
                line.append(d[f][c])
        return line



    def to_dict(self):
        """
        Dictionary representation of Report
        Example : {
                    "co01" :  {
                                "male": 120
                                "female": 213
                                "total": 333
                              },
                    "enters": {
                                "male": 120
                                "female": 213
                                "total": 333
                              },
                    "exits": {
                                "male": 120
                                "female": 213
                                "total": 333
                              },
                    "total":  999
                   }
        :return:
        """
        result = OrderedDict()
        result['total'] = self.male['total'] + self.female['total']
        enters_total = self.male['enters'] + self.female['enters']
        exits_total = self.male['exits'] + self.female['exits']
        result['enters'] = self.generate_line('enters', enters_total)
        result['exits'] = self.generate_line('exits', exits_total)
        for indicator in self.indicators:
            indicatortotal = self.male[indicator] + self.female[indicator]
            result[indicator] = self.generate_line(indicator, indicatortotal)

        return result

    def generate_line(self, key, itotal):
        """
        Generates line for report
        :param key: key for indicator, e.g co01
        :param itotal: relative total to this indicator
        :return: dictionary representing male, female and total values
        """
        return OrderedDict({
            "male": self.male[key],
            "female": self.female[key],
            "total": itotal
        })

    def empty_default(self):
        """
        generates a default with every indicator defaulting to 0
        :return:
        """
        return defaultdict.fromkeys(self.indicators + ['enters', 'exits', 'total'], 0)

    def determine_values_for_genders(self, rows):
        """
        Determines the rows for both gender
        In case no values are present, a default is generated
        :param rows:
        :return:
        """
        male = self.determine_values_for("male", rows)
        female = self.determine_values_for("female", rows)
        return female, male

    def determine_values_for(self, gender, rows):
        """
        If a gender is
        :param gender:
        :param rows:
        :return:
        """
        for row in rows:
            if row['gender'] == gender:
                return row
        return self.empty_default()
